<?php

class nmPremium {

    private static $instance;
    
    private $is_installed = false;
    private $is_enabled = false;
    private $controller;

    /**
     * Конструктор
     */
    private function __construct() {

        if (!cmsCore::isControllerExists('nm_premium')) { return; }

        $this->controller = cmsCore::getController('nm_premium');

        if (!$this->controller->isControllerInstalled('nm_premium')) { return; }

        $this->is_installed = true;

        if (!$this->controller->isEnabled()) { return; }

        $this->is_enabled = true;

    }

    /**
     * Экземпляр класса
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Установка контекста
     */
    public function setRequest(cmsRequest $request) {

        if (!$this->is_enabled) { return; }
        
        return $this->controller->setRequest($request);

    }

    /**
     * Добавление премиум скриптов и стилей на страницу
     */
    public function addToPage() {

        if (!$this->is_enabled) { return; }

        return $this->controller->addToPage();

    }

    /**
     * Запуск хука
     */
    public function runHook($event_name, $params = array(), $default = null) {

        if (!$this->is_enabled) { return; }

        $this->controller->runHook($event_name, $params, $default);

    }

    /**
     * Установлен ли премиум компонент
     */
    public function isInstalled() {

        return $this->is_installed;

    }

    /**
     * Включен ли премиум компонент
     */
    public function isEnabled() {

        return $this->is_enabled;

    }

    /**
     * Существует ли нужный экшен
     */
    public function isActionExists($action_name) {
        
        if (!$this->is_enabled) { return false; }

        return $this->controller->isActionExists($action_name);
    
    }

    /**
     * Запуск премиум экшена
     */
    public function runAction($action_name, $params = array()) {
    
        if (!$this->is_enabled) { return false; }

        return $this->controller->runAction($action_name, $params);
    
    }

    public function getController() {

        if (!$this->is_enabled) { return false; }

        return $this->controller;

    }

}