<?php

class modelNeomessenger extends cmsModel {

    /**
     * Извлечение сообщений из базы от определенного контакта
     * @param $user_id
     * @param $contact_id
     * @return array|bool
     */
    public function getMessages($user_id, $contact_id) {

        $this->select('u.nickname', 'user_nickname');
        $this->select('u.avatar', 'user_avatar');
        $this->join('{users}', 'u', 'u.id = i.from_id');

        if ($this->filter_on) { $this->filterAnd(); }

        $this->filterStart();

        $this->filterEqual('from_id', $user_id);
        $this->filterEqual('to_id', $contact_id);
        $this->filterIsNull('is_deleted_from');

        $this->filterOr();

        $this->filterEqual('from_id', $contact_id);
        $this->filterEqual('to_id', $user_id);
        $this->filterIsNull('is_deleted_to');

        $this->filterEnd();

        $this->filterIsNull('is_deleted');

        $this->orderBy('id', 'desc');

        $messages = $this->get('{users}_messages', function($message, $model) {
            return new nmMessage($message);
        }, false);

        return is_array($messages) ? array_reverse($messages) : false;

    }

    /**
     * Извлечение сообщений из базы от всех контактов
     * @param $user_id
     * @return array
     */
    public function getMessagesFromAllContacts($user_id) {

        $this->select('u.nickname', 'user_nickname');
        $this->select('u.avatar', 'user_avatar');
        $this->join('{users}', 'u', 'u.id = i.from_id');

        if ($this->filter_on) { $this->filterAnd(); }

        $this->filterStart();
        $this->filterEqual('to_id', $user_id);
        $this->filterIsNull('is_deleted_to');
        $this->filterIsNull('is_deleted');
        $this->filterEnd();

        $this->orderBy('id');

        return $this->get('{users}_messages', function($message, $model) {
            return new nmMessage($message);
        }, false);

    }

    /**
     * Получение списка контактов
     * @param $user_id
     * @return array
     */
    public function getContacts($user_id) {

        $this->select('u.id', 'id');
        $this->select('u.nickname', 'nickname');
        $this->select('u.avatar', 'user_avatar');
        $this->select('u.date_log', 'date_log');
        $this->select('u.is_admin', 'is_admin');
        $this->select('IFNULL(COUNT(m.id), 0)', 'new_messages');

        $this->join('{users}', 'u', 'u.id = i.contact_id');
        $this->joinLeft('{users}_messages', 'm', 'm.from_id = i.contact_id AND m.to_id = i.user_id AND m.is_new = 1 AND m.is_deleted_to IS NULL');
        $this->joinSessionsOnline();
        $this->filterEqual('user_id', $user_id);

        $this->groupBy('i.contact_id');

        $this->orderBy('date_last_msg', 'desc');

        return $this->get('{users}_contacts', function($contact, $model) {
            return new nmContact($contact, $model);
        }, false);

    }

    public function getContact($user_id, $contact_id) {
        
        $this->select('u.id', 'id');
        $this->select('u.nickname', 'nickname');
        $this->select('u.avatar', 'user_avatar');
        $this->select('u.date_log', 'date_log');
        $this->select('u.avatar', 'avatar');
        $this->select('u.is_admin', 'is_admin');
        $this->select('u.privacy_options', 'privacy_options');
        $this->select('COUNT(g.user_id)', 'is_ignored');
        $this->join('{users}', 'u', 'u.id = i.contact_id');
        $this->joinSessionsOnline();
        $this->joinLeft('{users}_ignors', 'g', "g.ignored_user_id = i.contact_id AND g.user_id = '{$user_id}'");

        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('contact_id', $contact_id);

        $this->groupBy('i.id');

        return $this->getItem('{users}_contacts', function($contact, $model) {
            return new nmContact($contact, $model);
        });

    }

    /**
     * В игноре ли контакт
     * @param $user_id
     * @param $contact_id
     * @return int
     */
    public function isContactIgnored($user_id, $contact_id) {

        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('ignored_user_id', $contact_id);
        $this->limit(1);

        $is_ignored = $this->getCount('{users}_ignors');

        $this->resetFilters();

        return $is_ignored;

    }

    /**
     * ID последнего сообщения
     * @param $user_id
     * @return int
     */
    public function getLastMessageID($user_id) {

        $this->filterEqual('to_id', $user_id);
        $this->filterIsNull('is_deleted_to');
        $this->filterIsNull('is_deleted');

        $this->orderBy('id', 'desc');

        return (int)$this->getFieldFiltered('{users}_messages', 'id');

    }

    /**
     * Количество новых сообщений
     * @param $user_id
     * @return int
     */
    public function getNewMessagesCount($user_id) {

        $this->filterEqual('to_id', $user_id);
        $this->filterEqual('is_new', 1);
        $this->filterIsNull('is_deleted_to');
        $this->filterIsNull('is_deleted');

        $count = $this->getCount('{users}_messages');

        return $count;

    }

    /**
     * ID последнего не прочитанного сообщения
     * @param $user_id
     * @return int
     */
    public function getLastUnreadMessageId($user_id) {

        $this->filterEqual('is_new', 1);

        return $this->getLastMessageID($user_id);

    }

    /**
     * Удаление контроллера
     * @param $id
     * @return bool
     */
    public function deleteController($id) {

        parent::deleteController($id);

        $this->filterEqual('controller', 'neomessenger')->deleteFiltered('scheduler_tasks');

        return true;

    }

}